from collections import OrderedDict
from .table import BaseTable


class ReplicationTable(BaseTable):
    __formatter_name__ = 'replication'

    def __init__(self, max_lines):
        header = OrderedDict()
        header['Host'] = 15
        header['Port'] = 5
        header['State'] = 9
        header['Last Update'] = 19
        header['Delay'] = 7
        header['Health'] = 5
        header['Priority'] = 5
        header['Votes'] = 5
        header['Hidden'] = 5

        super(ReplicationTable, self).__init__(header, max_lines)



import time


class BaseCommand(object):
    __command_name__ = ''

    def __init__(self, database_connection, command_options, filters=None):
        self._filters = filters
        self._database_connection = database_connection
        self._refresh = command_options['refresh']

    def start(self):
        start_time = time.time()

        while True:
            yield self._do()
            time.sleep(self._refresh - ((time.time() - start_time) % 1.0))

    def _do(self):
        raise NotImplementedError()



# -*- coding: utf-8 -*-
import argparse
from khan.mongodb_client import MongoDB
from khan.commands import command_factory
from khan.formatter import formatter_factory


def arg_parse():
    parser = argparse.ArgumentParser(
        add_help=True,
        description="Read the docs: https://github.com/globocom/khan"
    )

    parser.add_argument(
        '--host',
        help="MongoDB Host/instance",
        required=True
    )
    parser.add_argument(
        '--port',
        help="MongoDB port",
        required=True
    )
    parser.add_argument(
        '--database',
        help="Database name",
        required=True
    )
    parser.add_argument(
        "-u", dest='user',
        help="Database user",
        required=True
    )
    parser.add_argument(
        "-p", dest='password',
        help="Database password",
        required=True
    )
    parser.add_argument(
        "-r", dest='refresh',
        help="Refresh (seconds)",
        default=1,
        type=int,
        required=False
    )
    parser.add_argument(
        "-m", dest='method',
        choices=[
            "queries",
            "replication"],
        help="Show <method> status",
        action="store"
    )
    parser.add_argument(
        "-f", dest='filters',
        help="Dictionary with key/column and value/contains. "
             "E.g: \"{'Op': 'query', 'collection': 'db.users'}\". "
             "Only to queries method",
        required=False, default={}
    )
    parser.add_argument(
        "-l", dest='max_lines', type=int,
        help="Max of lines in table",
        required=False, default=30
    )
    return parser.parse_args()


def main():
    parameters = arg_parse()

    connection = MongoDB(
        host=parameters.host, port=parameters.host,
        database=parameters.database, user=parameters.user,
        password=parameters.password
    )

    command_options = {
        'refresh': parameters.refresh,
    }

    if parameters.filters:
        import ast
        parameters.filters = ast.literal_eval(parameters.filters)

    command_class = command_factory(parameters.method)
    command = command_class(connection, command_options, parameters.filters)

    table_class = formatter_factory(parameters.method)
    table = table_class(parameters.max_lines)

    try:
        for status in command.start():
            table.show(status)
    except KeyboardInterrupt:
        pass


if __name__ == '__main__':
    main()



from operator import itemgetter
from collections import OrderedDict
from .base import BaseCommand


class TopCommand(BaseCommand):
    __command_name__ = 'queries'

    def _sort_operations_by(self, operations, field):
        return sorted(operations, key=itemgetter(field), reverse=True)

    def _is_waiting_for_lock(self, operation):
        return 'Yes' if operation["waitingForLock"] else 'No'

    def _get_duration_in_seconds(self, operation):
        return str(int(operation['microsecs_running'] / 1000000)) + "s"

    def _get_operations(self):
        operations = self._database_connection.current_op()['inprog']
        for operation in  operations:
            if 'microsecs_running' not in operation:
                operation['microsecs_running'] = 0
        return self._sort_operations_by(operations, 'microsecs_running')

    def _build_reponse_dict(self, operation, line, query):
        current = OrderedDict()
        current["Id"] = operation["connectionId"]
        current["Op"] = operation["op"]
        current["wfl"] = self._is_waiting_for_lock(operation)
        current["Yields"] = operation["numYields"]
        current["Duration"] = self._get_duration_in_seconds(operation)
        current["collection"] = operation["ns"]
        current["query"] = query
        return current

    def _do(self):
        result = []
        for line, operation in enumerate(self._get_operations()):
            query = "NULL"
            if "insert" in operation:
                query = str(operation["insert"])
            elif "query" in operation:
                query = str(operation["query"])

            current = self._build_reponse_dict(operation, line, query)
            if self.is_in_filter(current):
                result.append(current)

        return result

    def is_in_filter(self, values):
        if not self._filters:
            return True

        if not all(filter in values.keys() for filter in self._filters.keys()):
            raise AttributeError(
                "{} don't have {}".format(values.keys(), self._filters.keys())
            )

        return all(
            (self._filters[key] in values[key] for key in self._filters.keys())
        )



from __future__ import print_function
from datetime import datetime as dt
import time
from prettytable import PrettyTable


DATE_TIME_FORMAT = '%d-%m-%Y %H:%M:%S'


class BaseTable(object):
    def __init__(self, header, max_lines=30):
        self.header = header
        self.max_lines = max_lines

    def show(self, lines):
        print(self.clear_shell(), end="")

        print(self.format_header())

        table = PrettyTable()
        table.field_names = self.header.keys()
        table.align = "l"

        for line in lines:
            table_line = 0
            while True:
                values = []
                for key in line.keys():
                    values.append(self.format_column(key, line[key], table_line))

                if not any(values):
                    break

                table.add_row(values)
                table_line += 1

            if len(table._rows) >= self.max_lines:
                break

        print(table)


    def format_column(self, key, info, table_line):
        if isinstance(info, dt):
            info = info.strftime(DATE_TIME_FORMAT)
        info = str(info)

        if key in self.header:
            diff = self.header[key] - len(info)
            if diff > 0:
                info += ' ' * diff

            current_size = self.header[key] * table_line
            return str(info)[current_size:current_size + self.header[key]]

        return ''


    def format_header(self):
        st = dt.fromtimestamp(time.time()).strftime(DATE_TIME_FORMAT)
        return "### {} ###".format(st)


    def clear_shell(self):
        return "\x1b[H\x1b[2J"



import sys
import inspect
from .replication import ReplicationCommand
from .top import TopCommand


def command_factory(command_name):
    klasses = inspect.getmembers(sys.modules[__name__], inspect.isclass)
    for klass in klasses:
        if klass[1].__command_name__ == command_name:
            return klass[1]
    else:
        raise AttributeError('Unknown method: {}'.format(command_name))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import logging
import pymongo

LOG = logging.getLogger(__name__)

MONGO_CONNECTION_DEFAULT_TIMEOUT = 5000


class AuthenticationError(Exception):
    pass


class ConnectionError(Exception):
    pass


class MongoDB(object):
    def __init__(self, host, port, database,
                 user, password, timeout=MONGO_CONNECTION_DEFAULT_TIMEOUT,
                 mongodb_client=pymongo.MongoClient):
        self._host = host
        self._port = port
        self._database = database
        self._user = user
        self._password = password
        self._timeout = timeout
        self._mongodb_client = mongodb_client
        self._authenticate()

    @property
    def _connection_string(self):
        return 'mongodb://{user}:{password}@{host}'.format(
            user=self._user, password=self._password, host=self._host)

    def _init_database_client(self):
        client = getattr(self._mongodb_client(
            self._connection_string,
            connectTimeoutMS=self._timeout

        ), self._database)

        return client

    def _authenticate(self):
        self._client = self._init_database_client()
        try:
            return self._client.authenticate(self._user, self._password)
        except pymongo.errors.OperationFailure as e:
            if e.code == 18:
                error_message = 'Invalid credentials to database {}: {}'.format(
                    self._database, self._connection_string)
                raise AuthenticationError(error_message)

    def _execute(self, func, attempts=10, command=None):
        try:
            return func(command)
        except pymongo.errors.PyMongoError as e:
            if attempts > 0:
                print('Reconnecting... Attemps: {}'.format(attempts))
                self._authenticate()
                return self._execute(func, attempts - 1, command)
            else:
                error_message = 'Error connecting to database {}: {}\n{}'.format(
                    self._database, self._connection_string, e)
                raise ConnectionError(error_message)

    def current_op(self):
        return self._execute(self._client.current_op)

    def execute_command(self, command):
        return self._execute(self._client.command, command=command)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup
from khan import __version__, __author__, __email__

with open('README.md') as readme_file:
    readme = readme_file.read()

requirements = [req[:-1] for req in open('requirements.txt', 'r').readlines()]

test_requirements = requirements

setup(
    name='khan-mongo',
    version=__version__,
    description="'hack day'",
    long_description=readme,
    author=__author__,
    author_email=__email__,
    url='https://github.com/otherpirate/khan',
    packages=[
        'khan',
    ],
    package_dir={'khan':
                 'khan'},
    include_package_data=True,
    install_requires=requirements,
    entry_points={
        'console_scripts': [
            'khan=khan.main:main',
        ],
    },
    license="MIT license",
    zip_safe=False,
    keywords='khan',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



from __future__ import print_function
from dateutil import tz
from collections import OrderedDict
from .base import BaseCommand


class ReplicationCommand(BaseCommand):
    __command_name__ = 'replication'

    def _do(self):
        result = []

        replSetGetStatus = self._database_connection.execute_command('replSetGetStatus')
        replSetGetConfig = self._database_connection.execute_command('replSetGetConfig')

        config_members = {}
        for member in replSetGetConfig['config']['members']:
            config_members[member['host']] = member

        optimeDate_Primary = None
        for member in replSetGetStatus['members']:
            if member['stateStr'] == 'PRIMARY':
                optimeDate_Primary = member['optimeDate'].replace(tzinfo=tz.tzutc()).astimezone(tz.tzlocal())

        for member in replSetGetStatus['members']:
            name = member['name']
            host, port = name.split(":")
            state = member['stateStr']
            if member.get('health', 1) == 1:
                health = 'Up'
            else:
                health = 'Down'

            delay = '-'
            optimeDate = '-'
            if state != 'ARBITER':
                optimeDate = member['optimeDate'].replace(tzinfo=tz.tzutc()).astimezone(tz.tzlocal())
                if optimeDate_Primary:
                    delay = optimeDate_Primary - optimeDate

            priority = config_members[name]['priority']
            votes = config_members[name]['votes']
            hidden = config_members[name]['hidden']

            current = OrderedDict()
            current["Host"] = host
            current["Port"] = port
            current["State"] = state
            current["Last Update"] = optimeDate
            current["Delay"] = delay
            current["Health"] = health
            current["Priority"] = priority
            current["Votes"] = votes
            current["Hidden"] = hidden
            result.append(current)

        return result



from collections import OrderedDict
from .table import BaseTable


class TopTable(BaseTable):
    __formatter_name__ = 'queries'

    def __init__(self, max_lines):
        header = OrderedDict()
        header['Id'] = 8
        header['Op'] = 8
        header['wfl'] = 3
        header['Yields'] = 6
        header['Duration'] = 8
        header['collection'] = 25
        header['query'] = 75

        super(TopTable, self).__init__(header, max_lines)
